$input v_color0, v_texcoord0, v_illum, v_uv

#include <bgfx_shader.sh>

SAMPLER2D(s_texture, 0);
SAMPLER2D(s_lighting_texture, 1);
SAMPLER2D(s_main_depth, 2);
uniform vec4 lightingData;
uniform vec2 subtile;
uniform vec4 depthThreshold;

void main()
{
    float targetDepth = texture2D(s_main_depth, v_texcoord0).x;
    if (targetDepth >= depthThreshold.x) {
        discard;
    }

    vec2 lightingUV = v_texcoord0 * lightingData.zw + lightingData.xy;
    vec4 lightColor = texture2D(s_lighting_texture, lightingUV).rgba;
    vec4 bloomColor = texture2D(s_texture, v_texcoord0);

    // Null the color
    lightColor.r = 0.0;
    lightColor.g = 0.0;
    lightColor.b = 0.0;
    lightColor.a = 0.0;

    // Sampling!
    float range = 8.0;
    float sampleCount = 0.0;
    vec2 startPos = vec2(lightingUV.x - range * subtile.x, lightingUV.y - range * subtile.y);
    for(float x = 0; x < range * 2; ++x)
    {
        for(float y = 0; y < range * 2; ++y)
        {
            vec2 samplePos = vec2(startPos.x + subtile.x * x, startPos.y + subtile.y * y);
            sampleCount += 1.0;
            lightColor += texture2D(s_lighting_texture, samplePos);
        }
    }

    // Average the samples
    lightColor.r /= sampleCount;
    lightColor.g /= sampleCount;
    lightColor.b /= sampleCount;
    lightColor.a /= sampleCount;

    // gl_FragColor = bloomColor * lightColor;
    bloomColor.rgb *= lightColor.x;
    gl_FragColor = bloomColor;
}
