$input v_color0, v_texcoord0, v_illum, v_uv

#include <bgfx_shader.sh>

SAMPLER2D(s_texture, 0);
SAMPLER2D(s_main_texture, 1);
SAMPLER2D(s_illum_texture, 2);
SAMPLER2D(s_main_depth, 3);
uniform vec4 lightingData;
uniform vec2 subtile;
uniform vec4 depthThreshold;

vec3 SaturationBrightness(vec3 color, float brt, float sat)
{
    // Increase or decrease theese values to adjust r, g and b color channels seperately
    const float AvgLumR = 0.5;
    const float AvgLumG = 0.5;
    const float AvgLumB = 0.5;

    const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);

    vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
    vec3 brtColor = color * brt;
    float i = dot(brtColor, LumCoeff);
    vec3 intensity = vec3(i, i, i);
    vec3 satColor = mix(intensity, brtColor, sat);
    return satColor;
}

float Average(vec4 color)
{
    return (color.r + color.g + color.b) / 3.0;
}

vec4 DarknessColor(vec4 color, float amount)
{
    vec4 desaturatedColor;
    vec4 resultColor;

    desaturatedColor.r = Average(color);
    desaturatedColor.g = desaturatedColor.r;
    desaturatedColor.b = desaturatedColor.r;
    desaturatedColor.a = color.a;

    desaturatedColor.r *= 0.7;
    desaturatedColor.g *= 1.1;
    desaturatedColor.b *= 1.3;

    return desaturatedColor * amount + color * (1.0 - amount);
}

void main()
{
    float targetDepth = texture2D(s_main_depth, v_texcoord0).x;
    if (targetDepth >= depthThreshold.x) {
        discard;
    }

    vec2 lightingUV = v_texcoord0 * lightingData.zw + lightingData.xy;
    vec4 lightColor = texture2D(s_texture, lightingUV).rgba;
    vec4 targetColor = texture2D(s_main_texture, v_texcoord0);
    vec4 illumColor = texture2D(s_illum_texture, v_texcoord0);

    // Null the color
    lightColor.r = 0.0;
    lightColor.g = 0.0;
    lightColor.b = 0.0;
    lightColor.a = 0.0;

    // Sampling!
    float range = 8.0;
    float sampleCount = 0.0;
    vec2 startPos = vec2(lightingUV.x - range * subtile.x, lightingUV.y - range * subtile.y);
    for(float x = 0; x < range * 2; ++x)
    {
        for(float y = 0; y < range * 2; ++y)
        {
            vec2 samplePos = vec2(startPos.x + subtile.x * x, startPos.y + subtile.y * y);
            sampleCount += 1.0;
            lightColor += texture2D(s_texture, samplePos);
        }
    }

    // Average the samples
    lightColor.r /= sampleCount;
    lightColor.g /= sampleCount;
    lightColor.b /= sampleCount;
    lightColor.a /= sampleCount;

    gl_FragColor = vec4(SaturationBrightness(lightColor.rgb, 1.1, 1.1), lightColor.a) * targetColor;
    //float avgColor = Average(lightColor);

    // Figure out how much we want to desaturate
    // if( avgColor < 0.5)
    // {
    //     float desatAmount = (1.0 - avgColor / 0.5);
    //     gl_FragColor = DarknessColor(gl_FragColor, desatAmount);
    // }

    // if (bloomColor.a > 0.1) {
    //     bloomColor.a = 0;
    // }

    // if (illumColor.r == 0.0) {
    //     // dvec4 combined = clamp(vec4(gl_FragColor.rgb + bloomColor.rgb, gl_FragColor.a), 0.0, 1.0);
    //     // gl_FragColor = mix(gl_FragColor, bloomColor, bloomColor.a);
    //     gl_FragColor = vec4(clamp(gl_FragColor.rgb + bloomColor.rgb, 0.0, 1.0), gl_FragColor.a);
    // } else {
    //     gl_FragColor = mix(gl_FragColor, targetColor, illumColor.r);
    // }
    // gl_FragColor = clamp(vec4(gl_FragColor.rgb + bloomColor.rgb, gl_FragColor.a + (lightColor.a * bloomColor.a * 2)), 0.0, 1.0);
}
