$input v_color0, v_texcoord0, v_illum, v_uv

#include <bgfx_shader.sh>

SAMPLER2D(s_texture, 0);
SAMPLER2D(s_previousColormap, 1);
SAMPLER2D(s_currentColormap, 2);
uniform vec4 time; // x (hours fraction), y (minutes fraction), z (hours in day), w (transition 0 = all previous, 1 = all current)

void main()
{
    // v_color0 (.r is current time color index, .g is next time color index, .b is the texture size)
    // We calculate the multiplier that is used to calculate the texture coordinates using the time color indices
    float multiplier = 255 / (v_color0.b * 255);

    vec2 currentTime = vec2(time.x, v_color0.r * multiplier);
    vec2 nextTime = vec2(time.z, v_color0.r * multiplier);

    vec4 previousTopColor = mix(texture2D(s_previousColormap, currentTime), texture2D(s_previousColormap, nextTime), time.y);
    vec4 currentTopColor = mix(texture2D(s_currentColormap, currentTime), texture2D(s_currentColormap, nextTime), time.y);
    vec4 topColor = mix(previousTopColor, currentTopColor, time.w);

    currentTime.y = v_color0.g * multiplier;
    nextTime.y = v_color0.g * multiplier;
    vec4 previousBottomColor = mix(texture2D(s_previousColormap, currentTime), texture2D(s_previousColormap, nextTime), time.y);
    vec4 currentBottomColor = mix(texture2D(s_currentColormap, currentTime), texture2D(s_currentColormap, nextTime), time.y);
    vec4 bottomColor = mix(previousBottomColor, currentBottomColor, time.w);

    gl_FragColor = mix(topColor, bottomColor, v_texcoord0.y);
}
